@@ -10,9 +10,22 @@ This module provides float characteristics that are usable directly as attribute
 
 """
 
+from __future__ import annotations
+
 from . import Attribute
 from . import StructCharacteristic
 
+try:
+    from typing import Optional, Type, Union, TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        from circuitpython_typing import ReadableBuffer
+        from adafruit_ble.uuid import UUID
+        from adafruit_ble.services import Service
+
+except ImportError:
+    pass
+
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_BLE.git"
 
@@ -23,12 +36,12 @@ class FloatCharacteristic(StructCharacteristic):
     def __init__(
         self,
         *,
-        uuid=None,
-        properties=0,
-        read_perm=Attribute.OPEN,
-        write_perm=Attribute.OPEN,
-        initial_value=None
-    ):
+        uuid: Optional[UUID] = None,
+        properties: int = 0,
+        read_perm: int = Attribute.OPEN,
+        write_perm: int = Attribute.OPEN,
+        initial_value: Optional[ReadableBuffer] = None,
+    ) -> None:
         if initial_value is not None:
             initial_value = (initial_value,)
         super().__init__(
@@ -40,10 +53,12 @@ class FloatCharacteristic(StructCharacteristic):
             initial_value=initial_value,
         )
 
-    def __get__(self, obj, cls=None):
+    def __get__(
+        self, obj: Optional[Service], cls: Optional[Type[Service]] = None
+    ) -> Union[float, "FloatCharacteristic"]:
         if obj is None:
             return self
         return super().__get__(obj)[0]
 
-    def __set__(self, obj, value):
+    def __set__(self, obj: Service, value: float) -> None:
         super().__set__(obj, (value,))
